/* 
 * Copyright (C) 2006-2024 Registro.br. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
/* $Id$ */
#if USE_IP_MANAGEMENT
#include <memory>
#include <string>

#include "libepp_nicbr.H"

#include "IpNetworkCheckTest.H"
#include "IpNetworkCheck.H"
#include "FileUtil.H"
#include "IoException.H"
#include "XmlException.H"

#include "StrUtil.H"

LIBEPP_NICBR_NS_USE

CPPUNIT_TEST_SUITE_REGISTRATION(IpNetworkCheckTest);

IpNetworkCheckTest::IpNetworkCheckTest() {}

IpNetworkCheckTest::~IpNetworkCheckTest() {}

void IpNetworkCheckTest::setUp() {}

void IpNetworkCheckTest::tearDown() {}

void IpNetworkCheckTest::set_xml_template_test()
{
	string to_be_parsed = 
		"<?xml version='1.0' encoding='UTF-8' standalone='no'?>"
		"<epp xmlns='urn:ietf:params:xml:ns:epp-1.0' "
		"xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' "
		"xsi:schemaLocation='urn:ietf:params:xml:ns:epp-1.0 "
		"epp-1.0.xsd'>"
		"<command>"
		"<check>"
		"<ipnetwork:check "
		"xmlns:ipnetwork='urn:ietf:params:xml:ns:ipnetwork-1.0' "
		"xsi:schemaLocation='urn:ietf:params:xml:ns:ipnetwork-1.0 "
		"ipnetwork-1.0.xsd'>"
		"$(ipRanges)$"
		"</ipnetwork:check>"
		"</check>"
		"$(clTRID)$"
		"</command>"
		"</epp>";
  
	IpNetworkCheck ipNetworkCheck;
	IpNetworkCheckCmd *ipNetworkCheckCmd = ipNetworkCheck.get_command();
  
	IpRange ipRange;
	ipRange.set_version("v4");
	ipRange.set_ipBegin("192.168.0.0");
	ipRange.set_ipEnd("192.168.0.255");

	ipNetworkCheckCmd->insert_ipRange(ipRange);
	ipNetworkCheckCmd->set_clTRID("ABC-12345");
	ipNetworkCheck.set_xml_template(to_be_parsed);

	string expected = 
		"<?xml version='1.0' encoding='UTF-8' standalone='no'?>"
		"<epp xmlns='urn:ietf:params:xml:ns:epp-1.0' "
		"xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' "
		"xsi:schemaLocation='urn:ietf:params:xml:ns:epp-1.0 "
		"epp-1.0.xsd'>"
		"<command>"
		"<check>"
		"<ipnetwork:check "
		"xmlns:ipnetwork='urn:ietf:params:xml:ns:ipnetwork-1.0' "
		"xsi:schemaLocation='urn:ietf:params:xml:ns:ipnetwork-1.0 "
		"ipnetwork-1.0.xsd'>"
		"<ipnetwork:ipRange version='v4'>"
		"<ipnetwork:startAddress>192.168.0.0</ipnetwork:startAddress>"
		"<ipnetwork:endAddress>192.168.0.255</ipnetwork:endAddress>"
		"</ipnetwork:ipRange>"
		"</ipnetwork:check>"
		"</check>"
		"<clTRID>ABC-12345</clTRID>"
		"</command>"
		"</epp>";
  
	CPPUNIT_ASSERT_EQUAL(expected, ipNetworkCheck.get_xml());

	// tests reset method
	ipNetworkCheckCmd = ipNetworkCheck.get_command();
	ipNetworkCheckCmd->reset();
	CPPUNIT_ASSERT(ipNetworkCheckCmd->get_ipRanges().empty());
  
	// parent attribute
	CPPUNIT_ASSERT(ipNetworkCheckCmd->get_clTRID() == "");
}

void IpNetworkCheckTest::command_test()
{
	IpNetworkCheck ipNetworkCheck;
	IpNetworkCheckCmd *ipNetworkCheckCmd = ipNetworkCheck.get_command();

	IpRange ipRange;
	ipRange.set_version("v4");
	ipRange.set_ipBegin("192.168.0.0");
	ipRange.set_ipEnd("192.168.0.255");

	ipNetworkCheckCmd->insert_ipRange(ipRange);
	ipNetworkCheckCmd->set_clTRID("ABC-12345");

	bool exception_caught = false;
	try {
		string xml_template = 
			FileUtil::read_file("../docs/templates/ipnetwork_check.xml");
		ipNetworkCheck.set_xml_template(xml_template);

		DomParser parser;
		parser.enable_validation("../docs/schemas");
		parser.parse_command(ipNetworkCheck.get_xml());
	} catch (const IoException &e) {
		exception_caught = true;
		printf("\nIO Exception: code [%d] message [%s]",
		       e.get_code(), e.get_msg().c_str());
	} catch (const XmlException &e) {
		exception_caught = true;
		printf("\nXml Exception: code [%d] message [%s] low level message [%s]\n",
		       e.get_code(), e.get_msg().c_str(), e.get_low_level_msg().c_str());
	}

	CPPUNIT_ASSERT(!exception_caught);
}

void IpNetworkCheckTest::response_test()
{
	string expected = 
		"<?xml version='1.0' encoding='UTF-8' standalone='no'?>"
		"<epp xmlns='urn:ietf:params:xml:ns:epp-1.0' "
		"xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' "
		"xsi:schemaLocation='urn:ietf:params:xml:ns:epp-1.0 "
		"epp-1.0.xsd'>"
		"<response>"
		"<result code='1000'>"
		"<msg>Command completed successfully</msg>"
		"</result>"
		"<resData>"
		"<ipnetwork:chkData "
		"xmlns:ipnetwork='urn:ietf:params:xml:ns:ipnetwork-1.0' "
		"xsi:schemaLocation='urn:ietf:params:xml:ns:ipnetwork-1.0 "
		"ipnetwork-1.0.xsd'>"
		"<ipnetwork:cd>"
		"<ipnetwork:ipRange avail='0' version='v4'>"
		"<ipnetwork:startAddress>192.168.0.0</ipnetwork:startAddress>"
		"<ipnetwork:endAddress>192.168.0.255</ipnetwork:endAddress>"
		"</ipnetwork:ipRange>"
		"<ipnetwork:reason>In use</ipnetwork:reason>"
		"</ipnetwork:cd>"
		"</ipnetwork:chkData>"
		"</resData>"
		"<trID>"
		"<clTRID>ABC-12345</clTRID>"
		"<svTRID>54322-XYZ</svTRID>"
		"</trID>"
		"</response>"
		"</epp>";

	DomParser parser;
	parser.enable_validation("../docs/schemas");

	IpNetworkCheck ipNetworkCheck;
	CPPUNIT_ASSERT_NO_THROW(ipNetworkCheck.set_response(expected, &parser));
	IpNetworkCheckRsp *ipNetworkCheckRsp = ipNetworkCheck.get_response();

	map <Response::ResultCode, Response::ResultInfo> results;
	map <Response::ResultCode, Response::ResultInfo>::const_iterator r_it;
	results = ipNetworkCheckRsp->get_result_list();
	r_it = results.begin();

	CPPUNIT_ASSERT(r_it != results.end());
	CPPUNIT_ASSERT_EQUAL(Response::OK, r_it->first);

	string result_code =
		StrUtil::to_string("%d", (int)r_it->first);

	string response =
		"<?xml version='1.0' encoding='UTF-8' standalone='no'?>"
		"<epp xmlns='urn:ietf:params:xml:ns:epp-1.0' "
		"xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' "
		"xsi:schemaLocation='urn:ietf:params:xml:ns:epp-1.0 "
		"epp-1.0.xsd'>"
		"<response>"
		"<result code='1000'>"
		"<msg";
  
	string result_lang = ipNetworkCheckRsp->get_result_lang();
	if (result_lang != "en") {
		response += " lang='" + result_lang + "'";
	}
  
	response += ">" + r_it->second.msg + "</msg>"
		"</result>"
		"<resData>"
		"<ipnetwork:chkData "
		"xmlns:ipnetwork='urn:ietf:params:xml:ns:ipnetwork-1.0' "
		"xsi:schemaLocation='urn:ietf:params:xml:ns:ipnetwork-1.0 "
		"ipnetwork-1.0.xsd'>";

	vector<IpNetworkCheckRsp::CheckData> checkData = 
		ipNetworkCheckRsp->get_availability_list();

	for (unsigned int i = 0; i < checkData.size(); i++) {
		response += "<ipnetwork:cd>";
		response += "<ipnetwork:ipRange";
      
		if (checkData[i].ipRange.get_available() == true) {
			response += " avail='1'";
		} else {
			response += " avail='0'";
		}
    
		if (checkData[i].ipRange.get_version() != "") {
			response += " version='" + checkData[i].ipRange.get_version() + "'";
		}
    
		response += ">"
			"<ipnetwork:startAddress>" + checkData[i].ipRange.get_ipBegin() + 
			"</ipnetwork:startAddress>"
			"<ipnetwork:endAddress>" + checkData[i].ipRange.get_ipEnd() + 
			"</ipnetwork:endAddress>"
			"</ipnetwork:ipRange>";
    
		if (checkData[i].reason != "") {
			response += "<ipnetwork:reason";
			if (checkData[i].reason_lang != "") {
				response += " lang='" + checkData[i].reason_lang + "'";
			}
      
			response += ">" + checkData[i].reason + "</ipnetwork:reason>";
		}

		response += "</ipnetwork:cd>";
	}
    
	response += "</ipnetwork:chkData>"
		"</resData>"
		"<trID>"
		"<clTRID>" + ipNetworkCheckRsp->get_clTRID() + "</clTRID>"
		"<svTRID>" + ipNetworkCheckRsp->get_svTRID() + "</svTRID>"
		"</trID>"
		"</response>"
		"</epp>";

	CPPUNIT_ASSERT_EQUAL(expected, response);

	// tests reset method
	ipNetworkCheckRsp->reset();
	CPPUNIT_ASSERT(ipNetworkCheckRsp->get_availability_list().empty());
}
#endif //USE_IP_MANAGEMENT
